#
# green-fishing.R, 23 Jul 23
#
# Data from:
# Green Fuzzing: A Saturation-based Stopping Criterion using Vulnerability Prediction
# Stephan Lipp, Daniel Elsner, Severin Kacianka, Alexander Pretschner, Marcel Böhme, and Sebastian Banescu

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)


# Chao1 estimator
Chao1=function(df)
{
fcnt=count(df$Bug_ID)
f1=length(which(fcnt$freq == 1))
f2=length(which(fcnt$freq == 2))

return(data.frame(found=nrow(fcnt), expect=(nrow(df)-1)/nrow(df) * f1^2/(2*f2)))
}


bug_freq=function(df)
{
fcnt=count(df$Bug_ID)
bfreq=fcnt$freq[order(fcnt$freq, decreasing=TRUE)]

lines(bfreq, col=pal_col[df$fuz_col[1]])
}



gf=read.csv("fuzzer_crashes.csv", as.is=TRUE)

u_fuzzer=unique(gf$Fuzzer)
gf$fuz_col=as.numeric(mapvalues(gf$Fuzzer, u_fuzzer, 1:length(u_fuzzer)))

# Gif2png=subset(gf, Subject == "Gif2png")
Gif2png=subset(gf, Subject == "JasPer")

Gif_Chao1=ddply(Gif2png, .(Fuzzer), Chao1)

pal_col=rainbow(length(u_fuzzer))

# Occurrences of distinct faults for each fuzzer
plot(1, type="n", log="y",
	yaxs="i",
	xlim=c(1, 30), ylim=c(1, 900),
	xlab="Fault ranking (per fuzzer)", ylab="Occurrences")

legend("topright", legend=u_fuzzer,
                        fill=pal_col, border="white", bty="n")


d_ply(Gif2png, .(Fuzzer), bug_freq)

dev.copy(dev=png, file="fuzz-fault-count.png")
dev.off()


